# Grey Scale Mode Plugin
# Copywright 2008, jim.foltz@gmail.com
#
# Version 08-sep-12a
#
# 08-sep-12
#   * Revert context menu works on Groups as well as faces
#   * Added JF namespace
#
# Plugin modifies the in-model material colors to a greyscale value.
# Saves the original colors in an attribute dict. attached to the
# material itself.
#
# Right-click an a grey color to Revert that material to it's
# original color. Appears only in Grey Mode.

require 'sketchup'

menu = UI.menu("Plugins").add_submenu("Grey Scale")

cmd = UI::Command.new("Grey Mode") { JF::GreyScale::toGreyscale }
cmd.set_validation_proc { JF::GreyScale::greymode? }
menu.add_item(cmd)

cmd = UI::Command.new("Color Mode") { JF::GreyScale::revert }
cmd.set_validation_proc { JF::GreyScale::colormode? }
menu.add_item(cmd)

UI.add_context_menu_handler do |menu|
  sel = Sketchup.active_model.selection.first
  #if sel.typename == "Face" and ( material = sel.material )
  if JF::GreyScale.getMode == "grey"
    if  material = sel.material 
      rgb = material.get_attribute("GreyScaleColors", 'rgb')
      if rgb
	menu.add_item("Revert Color") do 
	  r, g, b = rgb.split(",").map{|e| e.to_i}
	  material.color = Sketchup::Color.new(r, g, b)
	end
      end
    end
  end
end

module JF
  module GreyScale; end

  #def GreyScale.mode
  #@mode
  #end

  def GreyScale.greymode?
    @mode = GreyScale.getMode
    if @mode == "grey"
      return MF_GRAYED
    end
    if Sketchup.active_model.materials.length == 0
      return MF_GRAYED
    end
  end

  def GreyScale.colormode?
    @mode = GreyScale.getMode
    return MF_GRAYED if @mode.nil?
    if @mode == "color"
      return MF_GRAYED
    end
    if Sketchup.active_model.materials.length == 0
      return MF_GRAYED
    end
  end

  def GreyScale.setMode(mode)
    Sketchup.active_model.set_attribute("GreyScale", "Mode", mode)
  end

  def GreyScale.getMode
    @mode = Sketchup.active_model.get_attribute("GreyScale", "Mode")
  end


  def GreyScale.calculateGreyscaleFromColor(color, type = :common)
    r = color.red
    g = color.green
    b = color.blue
    case type
    when :simple
      #Sketchup::Color.new( rand(255), rand(255), rand(255) )
      v = ( r + g + b ) / 3
      v = Integer( v )
      return Sketchup::Color.new( v, v, v )
    when :common
      # Greyscale = (r * 0.3) + (g * 0.59) + (b * 0.11)
      v =  r * 0.3 + g * 0.59 + b * 0.11  
      v = Integer( v )
      return Sketchup::Color.new( v, v, v )
    else
      puts "Bad GreyScale type: #{type}"
    end
  end


  # swap all to grey
  def GreyScale.toGreyscale
    #@allMaterials.each do |material|
    Sketchup.active_model.materials.each do |material|
      material.set_attribute("GreyScaleColors", "rgb", "#{material.color.red},#{material.color.green},#{material.color.blue}")
      material.color = GreyScale.calculateGreyscaleFromColor(material.color)
    end
    @mode = "grey"
    GreyScale.setMode(@mode)
  end

  def GreyScale.revert
    #@allMaterials.each { |material|
    Sketchup.active_model.materials.each do |material|
      next unless rgb = material.get_attribute("GreyScaleColors", 'rgb')
      r, g, b = rgb.split(",").map{|e| e.to_i}
      material.color = Sketchup::Color.new(r, g, b)
    end
    @mode = "color"
    GreyScale.setMode(@mode)
  end

end # module JF
